/* $Id: cryutils.h,v 1.4 1998/01/26 01:05:34 futo Exp $
 * Crypto-utils... Header
 *
 * Author: Ariel Futoransky
 */
#ifndef CRYUTILS
#define CRYUTILS
#include "compat.h"
	
#define CRYPTBUFSIZE 8			/* 8 Bytes (64-bit) blocksize for Blowfish */


int getrandomkey (char *, int *, int); /* (key, keylen, maxkey) */
int cflushout(int fd);
int cflushin(int fd);
int authpeer(int fd, u_int8_t *key, int keylen,  u_int32_t *data);
int cread(int fd, u_int8_t *buf, int len);
int cwrite(int fd, u_int8_t *buf, int len);
int cput_int32( int, u_int32_t );
int cget_int32( int );
int cput_str( int, char * );
int cget_str( int, char *, int);
void getrandom (int, char *, int );

#endif
